use rusqlite::Connection;
use rusqlite_migration::{Migrations, M};

pub struct MigrationRunner {}

impl MigrationRunner {
    pub fn up(connection: &mut Connection) -> bool {
        let m_create_events = include_str!("./1697409647688_create_events.sql");
        let m_event_seen_by_relays = include_str!("1706115586021_event_seen_by_relays.sql");
        let m_add_realys = include_str!("./1697410161900_add_relays.sql");
        let m_events_fts = include_str!("./1697410223576_events_fts.sql");
        let m_users = include_str!("./1697410294265_users.sql");
        let m_unattached_media = include_str!("./1697410480767_unattached_media.sql");
        let m_nip05 = include_str!("./1706575155557_nip05.sql");
        let m_nip42 = include_str!("./1707327016995_nip42_profile.sql");

        let migrations = Migrations::new(vec![
            M::up(m_create_events),
            M::up(m_event_seen_by_relays),
            M::up(m_add_realys),
            M::up(m_events_fts),
            M::up(m_users),
            M::up(m_unattached_media),
            M::up(m_nip05),
            M::up(m_nip42),
        ]);

        match migrations.to_latest(connection) {
            Ok(()) => true,
            Err(err) => {
                log::error!("Migrations failed: {}", err.to_string());
                false
            }
        }
    }
}

#[cfg(test)]
mod tests {
    use rusqlite::Connection;

    use super::MigrationRunner;

    #[test]
    fn get_sql_path() {
        let mut connection = Connection::open_in_memory().unwrap();

        let runner = MigrationRunner::up(&mut connection);

        assert!(runner);
    }
}
